// -------------------------------------------------------------------------
// PARAMETERS:
// -------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float Bright <string uiname="Brightness";>  = (0.f);

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};
// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// -------------------------------------------------------------------------
// PIXELSHADERS:
// -------------------------------------------------------------------------


float4 hello_01(vs2ps In): COLOR
 //float4 col = tex2D(Samp, In.TexCd);
{
    //texture lookup to access the pixels of the texture:
    float4 col = tex2D(Samp, In.TexCd);
    
    // First we do the brightness...
    // For Darker:
    //     X = X - (Value * X)
    if (Bright < 0)
    {
    // Use Absolute Brightness
    Bright = abs (Bright);
    col.rgb = col.rgb - (Bright * col.rgb) ;
    }
    // For Lighter:
    //     X = X + Value *(1 - x)
    else if (Bright > 0)
    {
    col.rgb = col.rgb + Bright * (1-col.rgb) ;
    }
    // Used formula for Gray:
    //gray=0.299*red+0.587*green+0.114*blue
    col.rgb = ((0.299* col.r) + (0.587 * col.g) + (0.114 *col.b));

    return col;
}
float4 hello_02(vs2ps In): COLOR
{
    //texture lookup to access the pixels of the texture:
    float4 col = tex2D(Samp, In.TexCd);
    // Brightness crap!!
    if (Bright < 0)
    {
    // Use Absolute Brightness
    Bright = abs (Bright);
    col.rgb = col.rgb - (Bright * col.rgb) ;
    }
    // For Lighter:
    //     X = X + Value *(1 - x)
    else if (Bright > 0)
    {
    col.rgb = col.rgb + Bright * (1-col.rgb) ;
    }
    // Used formula for Gray:
    //gray= ((red+green+blue) / 3 )
     col.rgb = ((col.r +col.g +col.b) / 3);
    return col;
}

// -------------------------------------------------------------------------
// TECHNIQUES:
// -------------------------------------------------------------------------

technique HelloInGrey1 //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 hello_01();
    }
}

technique HelloInGrey2 //name for the technique pin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 hello_02();
    }
}
